# Environ stand-alone programs

`driver.f90` to run custom calculations using Environ

The driver is a minimal program designed to call custom routines for custom Environ runs. <br> Users may report interest in additional custom functionality by providing calculation detail <br> to olivieroandreussi@unt.edu or by forking the Environ repo and submitting a pull request <br> for the custom code.

<br>

# Installation

## Environ compilation

The standalone programs require an Environ compilation. For details on how to compile <br> Environ, see the main [`README.md`](../README.md) file in the root directory.

## Stand-alone program compilation

With Environ compiled, from `Environ/programs`, run

    make driver

to compile the respective program.

<br>

# Using the driver

    make avail

to see available calculations.

    make usage name=name-of-calculation

for specific execution instructions.

<br>

# Calculation details

## `from_cube`

Computes environment contributions to properties of a "frozen" density given as a cube file

### Requirements

- Environ input file and cube file names may be provided as command-line arguments <br> (see calculation usage for details)

- Environ is unable to determine exactly the energy cutoff used in the calculation that <br> produced the cube file. Though an algorithm for slightly overestimating the cutoff <br> value is implemented in Environ, we recommend the user to provide the cutoff value <br> as `env_ecut` (Ry) in the &ENVIRON section of the input. This value can be obtained <br> from prior knowledge of the system, or by running convergence testing w.r.t total <br> energy difference

- atomic charges must be provided in the cube file header. These depend on the type of <br> calculation that produced the cube file. Replace `charge` with the atomic charge of the <br> corresponding species

  ```
  CUBE FILE GENERATED BY PW.X
  OUTER LOOP: X, MIDDLE LOOP: Y, INNER LOOP: Z
    3    0.000000    0.000000    0.000000
  120    0.166667    0.000000    0.000000
  120    0.000000    0.166667    0.000000
  120    0.000000    0.000000    0.166667
    8     charge    11.790000   12.050000   11.500000
    1     charge    13.450000   11.220000   11.500000
    1     charge    10.560000   10.660000   11.500000
  ```

## `descriptors`

Computes Environ descriptors based on total/partial surface and volume of the ionic boundary, <br> as well as their forces

### Requirements

- Environ input file, cube file name, and min/max/step radius values may be provided as <br> command-line arguments (see calculation usage for details)

- Environ is unable to determine exactly the energy cutoff used in the calculation that <br> produced the cube file. Though an algorithm for slightly overestimating the cutoff <br> value is implemented in Environ, we recommend the user to provide the cutoff value <br> as `env_ecut` (Ry) in the &ENVIRON section of the input. This value can be as small as <br> 10 depending on system size

- Environ is able to convert a cube file in Angstrom to internal units (a.u.) although the <br> output values will be given in a.u.. To do this, put -1 in place of the grid number in the <br> cube file with the grid spacing being the cell size. `charge` is not necessary for this type <br> of calculation, so zero can be put in place of charge

  ```
  CUBE FILE GENERATED BY PW.X
  OUTER LOOP: X, MIDDLE LOOP: Y, INNER LOOP: Z
    3    0.000000    0.000000    0.000000
    1   20.000000    0.000000    0.000000
    1    0.000000   20.000000    0.000000
    1    0.000000    0.000000   20.000000
    8    0.000000   11.790000   12.050000   11.500000
    1    0.000000   13.450000   11.220000   11.500000
    1    0.000000   10.560000   10.660000   11.500000

- The min/max/step values give the range of radii around all atoms for local values of <br> Surface and Volume and can be passed as command line arguments: <br> `-min #`, `-max #`, `-step #`

- The forces due to this ionic boundary can also be calculated by passing `--force` as <br> a command line argument
